==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/base64.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
"$Id: base64.c,v 1.6 2003/10/13 09:10:07 lwa Exp $";

#include "config.h"

#include <stdlib.h>
#include <string.h>
#include <errno.h>

static const unsigned char base64[] =
"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

char *base64_alloc_encode(char *string, size_t len) {
  char *res = malloc(len * 4 / 3 + 4);
  char *pos = res;
  unsigned char *ustring = string;
  if (res == NULL)
    return NULL;
  while(len > 0) {
    unsigned int acc = *ustring++; 
    acc <<= 8;
    if (len > 1) acc += *ustring++; 
    acc <<= 8;
    if (len > 2) acc += *ustring++; 
    *pos++ = base64[(acc & 0xfc0000) >> 18];
    *pos++ = base64[(acc & 0x03f000) >> 12];
    if (len > 1) 
      *pos++ = base64[(acc & 0x000fc0) >>  6];
    else
      *pos++ = '=';

    if (len > 2)
      *pos++ = base64[(acc & 0x00003f) >>  0];
    else
      *pos++ = '=';

    if (len <= 3)
      break;
    len -= 3;
  }
  *pos = 0;
  return res;
}

static int rbase64[128] = {
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
  -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
  -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
};

static int digit64(unsigned char c) {
  return c<=127 ? rbase64[c] : -1;
}

/*char *base64_decode(char *string, size_t ilen, size_t *olen) { */
size_t base64_decode(char *string, size_t ilen, unsigned char *res) {
  size_t i = 0;

  if (ilen%4 != 0) {
    errno = EILSEQ;
    return -1;
  }
  
  while(ilen > 0) {
    int d0, d1, d2, d3;
    
    if ( (d0 = digit64(string[0])) == -1 ||
	 (d1 = digit64(string[1])) == -1 ||
	 (((d2 = digit64(string[2])) == -1) && string[2] != '=') ||
	 (((d3 = digit64(string[3])) == -1) && string[3] != '=' )) {
      errno = EILSEQ;
      return -1;
    }
    res[i++] = (d0 << 2) | (d1 >> 4);
    if (d2 >= 0) {
      res[i++] = ((d1 << 4) & 0xf0) | (d2 >> 2);
      if (d3 >= 0) {
	res[i++] = ((d2 << 6 ) & 0xc0) | d3;
      } 
    }
    string += 4;
    ilen -= 4;
  }
  return i;
}

char *base64_alloc_decode(char *string, size_t *olen) { 
  char *res;
  size_t len;
  size_t ilen = strlen(string);
  if (ilen%4 != 0) {
    errno = EILSEQ;
    return NULL;
  }
  if ((res = malloc(ilen/4 * 3 + 2)) == NULL)
      return NULL;
  len = base64_decode(string, ilen, res);
  if (len == (size_t)-1) {
    free(res);
    return NULL;
  }
  res[len] = 0;
  if (olen)
    *olen = len;
  return res;
}


#if 0
#include <stdio.h>
#include <string.h>
int main(int argc, char **argv) {
  printf("%s\n", base64_alloc_decode(argv[1], NULL));
}
#endif

==> lmtpd-0.9.24.tar.gz: lmtpd-0.9.24/base64.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
"$Id: base64.c,v 1.7 2004/10/26 09:59:19 lwa Exp $";



#include <stdlib.h>
#include <string.h>
#include <errno.h>

static const unsigned char base64[] =
"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

char *base64_alloc_encode(char *string, size_t len) {
  char *res = malloc(len * 4 / 3 + 4);
  char *pos = res;
  unsigned char *ustring = string;
  if (res == NULL)
    return NULL;
  while(len > 0) {
    unsigned int acc = *ustring++; 
    acc <<= 8;
    if (len > 1) acc += *ustring++; 
    acc <<= 8;
    if (len > 2) acc += *ustring++; 
    *pos++ = base64[(acc & 0xfc0000) >> 18];
    *pos++ = base64[(acc & 0x03f000) >> 12];
    if (len > 1) 
      *pos++ = base64[(acc & 0x000fc0) >>  6];
    else
      *pos++ = '=';

    if (len > 2)
      *pos++ = base64[(acc & 0x00003f) >>  0];
    else
      *pos++ = '=';

    if (len <= 3)
      break;
    len -= 3;
  }
  *pos = 0;
  return res;
}

static int rbase64[128] = {
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,
  -1,  0,  1,  2,  3,  4,  5,  6,  7,  8,  9, 10, 11, 12, 13, 14,
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,
  -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,
};

static int digit64(unsigned char c) {
  return c<=127 ? rbase64[c] : -1;
}

/*char *base64_decode(char *string, size_t ilen, size_t *olen) { */
size_t base64_decode(char *string, size_t ilen, unsigned char *res) {
  size_t i = 0;

  if (ilen%4 != 0) {
    errno = EILSEQ;
    return -1;
  }
  
  while(ilen > 0) {
    int d0, d1, d2, d3;
    
    if ( (d0 = digit64(string[0])) == -1 ||
	 (d1 = digit64(string[1])) == -1 ||
	 (((d2 = digit64(string[2])) == -1) && string[2] != '=') ||
	 (((d3 = digit64(string[3])) == -1) && string[3] != '=' )) {
      errno = EILSEQ;
      return -1;
    }
    res[i++] = (d0 << 2) | (d1 >> 4);
    if (d2 >= 0) {
      res[i++] = ((d1 << 4) & 0xf0) | (d2 >> 2);
      if (d3 >= 0) {
	res[i++] = ((d2 << 6 ) & 0xc0) | d3;
      } 
    }
    string += 4;
    ilen -= 4;
  }
  return i;
}

char *base64_alloc_decode(char *string, size_t *olen) { 
  char *res;
  size_t len;
  size_t ilen = strlen(string);
  if (ilen%4 != 0) {
    errno = EILSEQ;
    return NULL;
  }
  if ((res = malloc(ilen/4 * 3 + 2)) == NULL)
      return NULL;
  len = base64_decode(string, ilen, res);
  if (len == (size_t)-1) {
    free(res);
    return NULL;
  }
  res[len] = 0;
  if (olen)
    *olen = len;
  return res;
}


#if 0
#include <stdio.h>
#include <string.h>
int main(int argc, char **argv) {
  printf("%s\n", base64_alloc_decode(argv[1], NULL));
}
#endif
